@@ -412,8 +412,11 @@ sub handle_request {
 
     local $Dispatcher = $self->new();
 
-HANDLER: {
+    eval {
         $Dispatcher->_do_dispatch($path);
+    };
+    if ( my $err = $@ ) {
+        $self->log->warn(ref($err) . " " ."'$err'") if ( $err !~ /^LAST RULE/);
     }
 }
 
@@ -626,24 +629,35 @@ sub _do_show {
 
     # Fix up the path
     $path = shift if (@_);
-    $path ||= request->path;
+    $path ||= $self->{path};
+    $self->log->debug("Showing path $path");
 
     # If we've got a working directory (from an "under" rule) and we have 
     # a relative path, prepend the working directory
     $path = "$self->{cwd}/$path" unless $path =~ m{^/};
 
     # If we're requesting a directory, go looking for the index.html
-    if ($path =~ m{/$} and 
-        Jifty->handler->mason->interp->comp_exists($path."/index.html")) {
-         $path .= "index.html" 
+    if ( $path =~ m{/$} and
+        Jifty->handler->mason->interp->comp_exists( $path . "/index.html" ) )
+    {
+        $path .= "/index.html";
     }
-    # Redirect to directory (and then index) if they requested 
+
+    # Redirect to directory (and then index) if they requested
     # the directory itself
-    
+
     # XXX TODO, we should search all component roots
-    $self->_do_redirect($path . "/") 
-      if -d Jifty::Util->absolute_path( (Jifty->config->framework('Web')->{'TemplateRoot'} || "html") . $path );
     
+    if ($path !~ m{/$}
+        and -d Jifty::Util->absolute_path(
+            Jifty->config->framework('Web')->{'TemplateRoot'} . $path
+        )
+        )
+    {
+
+        $self->_do_show( $path . "/" );
+    }
+
     # Set the request path
     request->path($path);
 
@@ -700,22 +714,19 @@ sub _do_dispatch {
 
     # Normalize the path.
     $self->{path} =~ s{/+}{/}g;
-    $self->{path} =~ s{/$}{};
 
     $self->log->debug("Dispatching request to ".$self->{path});
     eval {
-        HANDLER: {
-            $self->_handle_rules( [ $self->rules('SETUP') ] );
-          HANDLE_WEB: {
-                Jifty->web->handle_request();
-            }
-            $self->_handle_rules( [ $self->rules('RUN'), 'show' ] );
-            $self->_handle_rules( [ $self->rules('CLEANUP') ] );
-        }
+        $self->_handle_rules( [ $self->rules('SETUP') ] );
+        HANDLE_WEB: { Jifty->web->handle_request(); }
+        $self->_handle_rules( [ $self->rules('RUN'), 'show' ] );
+        $self->_handle_rules( [ $self->rules('CLEANUP') ] );
     };
     if ( my $err = $@ ) {
-        warn ref($err) . " " ."'$err'" if ( $err !~ /^LAST RULE/);
+        $self->log->warn(ref($err) . " " ."'$err'") if ( $err !~ /^LAST RULE/);
+
     }
+    last_rule;
 }
 
 =head2 _match CONDITION
@@ -833,7 +844,7 @@ sub _compile_condition {
     if ( $Dispatcher->{rule} eq 'on' ) {
 
         # "on" anchors on complete match only
-        $cond .= '\\z';
+        $cond .= '/?\\z';
     } else {
 
         # "in" anchors on prefix match in directory boundary
