@@ -5,7 +5,7 @@ package Jifty::Web::Form;
 
 use base qw/Jifty::Object Class::Accessor/;
 
-__PACKAGE__->mk_accessors(qw(actions printed_actions name call));
+__PACKAGE__->mk_accessors(qw(actions printed_actions name call is_open));
 
 =head2 new ARGS
 
@@ -75,6 +75,18 @@ Gets or sets the HTML name given to the form element.
 
 =cut
 
+=head2 is_open [BOOL]
+
+This accessor returns true if Jifty is currently in the middle of rendering a form 
+(if it's printed a <form> but not yet printed a </form> tag.) Use this in your 
+components to decide whether to open a form or not if you might be called from a 
+template that opened the form for you.
+
+=cut
+
+
+
+
 =head2 add_action PARAMHASH
 
 Calls L<Jifty::Web/new_action> with the paramhash given, and adds it to
@@ -91,7 +103,9 @@ sub add_action {
 
 =head2 register_action ACTION
 
-Adds C<ACTION> as an action for this form. Called so that actions' form fields can register the action against the form they're being used in.
+Adds C<ACTION> as an action for this form. Called so that actions'
+form fields can register the action against the form they're being
+used in.
 
 =cut
 
@@ -133,6 +147,11 @@ sub start {
     my $self = shift;
 
     my %args = (@_);
+
+    if ($self->is_open) {
+        Jifty->logger->warning("Trying to open a form when we already have one open");
+    }
+
     for (keys %args) {
         $self->$_($args{$_}) if $self->can($_);
     }
@@ -141,6 +160,7 @@ sub start {
     $form_start   .= qq! name="@{[ $self->name ]}"! if defined $self->name;
     $form_start   .= qq! enctype="multipart/form-data" >\n!;
     Jifty->web->out($form_start);
+    $self->is_open(1);
     '';
 } 
 
@@ -175,6 +195,9 @@ internal state such that L</start> may be called again.
 sub end {
     my $self = shift;
 
+    unless ($self->is_open) {
+        Jifty->logger->warning("Trying to close a form when we don't have one open");
+    }
     Jifty->web->out( qq!<div class="hidden">\n! );
 
     $self->_print_registered_actions();
@@ -184,7 +207,7 @@ sub end {
     Jifty->web->out( qq!</div>\n! );
 
     Jifty->web->out( qq!</form>\n! );
-
+    $self->is_open(0);
     # Clear out all the registered actions and the name 
     $self->_init();
 
@@ -214,8 +237,9 @@ sub print_action_registration {
 }
 
 
-# At the point this is called, it should only include actions we're registering that have no form fields
-# and haven't been explicitly registered.
+# At the point this is called, it should only include actions we're
+# registering that have no form fields and haven't been explicitly
+# registered.
 sub _print_registered_actions {
     my $self = shift;
     for my $a ( keys %{ $self->actions } ) {
